@@ -76,14 +76,10 @@ class TestObjectFactory(unittest.TestCase):
         project_root = find_project_root(__file__)
         cls._context = Context(name="DEFAULT")
 
-        cls._connection = Connection(server='http://localhost:7200',
-                                     repo="oldap",
-                                     userId="rosenth",
+        cls._connection = Connection(userId="rosenth",
                                      credentials="RioGrande",
                                      context_name="DEFAULT")
-        cls._unpriv = Connection(server='http://localhost:7200',
-                                 repo="oldap",
-                                 userId="unknown",
+        cls._unpriv = Connection(userId="unknown",
                                  credentials="RioGrande",
                                  context_name="DEFAULT")
         cls._context['test'] = 'http://oldap.org/test#'
@@ -96,7 +92,11 @@ class TestObjectFactory(unittest.TestCase):
 
         file = project_root / 'oldaplib' / 'ontologies' / 'admin.trig'
         cls._connection.upload_turtle(file)
-        sleep(1)  # upload may take a while...
+        #sleep(1)  # upload may take a while...
+
+        file = project_root / 'oldaplib' / 'ontologies' / 'admin-testing.trig'
+        cls._connection.upload_turtle(file)
+        #sleep(1)  # upload may take a while...
 
         file = project_root / 'oldaplib' / 'testdata' / 'objectfactory_test.trig'
         cls._connection.upload_turtle(file)
@@ -326,6 +326,16 @@ class TestObjectFactory(unittest.TestCase):
         self.assertEqual(at.unsignedByteProp, at2.unsignedByteProp)
         self.assertEqual(at.positiveIntegerProp, at2.positiveIntegerProp)
 
+    def test_constructor_c(self):
+        factory = ResourceInstanceFactory(con=self._connection, project='test')
+        Book = factory.createObjectInstance('Book')
+
+        b = Book(title="History of Time",
+                 author=[Iri('test:AlbertEinstein', validate=False), Iri('test:JohnWick', validate=False)],
+                 pubDate="1995-09-27",
+                 grantsPermission=Iri('oldap:GenericView'))
+        b.create()
+
     def test_value_setter(self):
         factory = ResourceInstanceFactory(con=self._connection, project='test')
         SetterTester = factory.createObjectInstance('SetterTester')
@@ -405,6 +415,24 @@ class TestObjectFactory(unittest.TestCase):
         self.assertEqual(obj1.integerSetter, {Xsd_int(20), Xsd_int(42)})
         self.assertFalse(obj1.booleanSetter)
 
+    def test_value_maxcount_mincount(self):
+        factory = ResourceInstanceFactory(con=self._connection, project='test')
+        Person = factory.createObjectInstance('Person')
+        p = Person(familyName="Müller",
+                   givenName="Max",
+                   grantsPermission={Iri('oldap:GenericView'), Iri('oldap:GenericUpdate')})
+        p.create()
+        sleep(1)
+        obj1 = Person.read(con=self._connection,
+                           project='test',
+                           iri=p.iri)
+        with self.assertRaises(OldapErrorValue):
+            obj1.familyName.add("Meier")
+        with self.assertRaises(OldapErrorValue):
+            obj1.givenName.discard("Max")
+        with self.assertRaises(OldapErrorValue):
+            del obj1.familyName
+
     def test_value_modifier_norights(self):
         # ps = PermissionSet(con=self._connection,
         #                    permissionSetId="testNoUpdate",
@@ -434,9 +462,7 @@ class TestObjectFactory(unittest.TestCase):
                             grantsPermission={Iri('oldap:GenericView'), Iri('oldap:GenericUpdate'), self._tps.iri})
         obj1.create()
 
-        unpriv = Connection(server='http://localhost:7200',
-                            repo="oldap",
-                            userId="factorytestuser",
+        unpriv = Connection(userId="factorytestuser",
                             credentials="Waseliwas",
                             context_name="DEFAULT")
         factory = ResourceInstanceFactory(con=unpriv, project='test')
@@ -511,9 +537,7 @@ class TestObjectFactory(unittest.TestCase):
         b.grantsPermission.add('hyha:HyperHamletMember')
         b.update()
 
-        unpriv = Connection(server='http://localhost:7200',
-                            repo="oldap",
-                            userId="factorytestuser",
+        unpriv = Connection(userId="factorytestuser",
                             credentials="Waseliwas",
                             context_name="DEFAULT")
         factory = ResourceInstanceFactory(con=unpriv, project=project)
