@@ -44,15 +44,16 @@ class OldapListHelperTestCase(unittest.TestCase):
         cls._context = Context(name="DEFAULT")
         cls._context['test'] = NamespaceIRI("http://testing.org/datatypes#")
         cls._context.use('test')
-        cls._connection = Connection(server='http://localhost:7200',
-                                     repo="oldap",
-                                     userId="rosenth",
+        cls._connection = Connection(userId="rosenth",
                                      credentials="RioGrande",
                                      context_name="DEFAULT")
         cls._connection.clear_graph(Xsd_QName('oldap:admin'))
         file = project_root / 'oldaplib' / 'ontologies' / 'admin.trig'
         cls._connection.upload_turtle(file)
 
+        file = project_root / 'oldaplib' / 'ontologies' / 'admin-testing.trig'
+        cls._connection.upload_turtle(file)
+
 
         cls._connection.clear_graph(Xsd_QName('test:test'))
         cls._connection.clear_graph(Xsd_QName('test:onto'))
