@@ -20,10 +20,11 @@ from oldaplib.src.enums.resourceclassattr import ResClassAttribute
 from oldaplib.src.enums.xsd_datatypes import XsdDatatypes
 from oldaplib.src.helpers.context import Context
 from oldaplib.src.helpers.langstring import LangString
-from oldaplib.src.helpers.oldaperror import OldapErrorAlreadyExists, OldapErrorNoPermission
+from oldaplib.src.helpers.oldaperror import OldapErrorAlreadyExists, OldapErrorNoPermission, OldapErrorInUse
 from oldaplib.src.helpers.query_processor import QueryProcessor
 from oldaplib.src.helpers.semantic_version import SemanticVersion
 from oldaplib.src.helpers.serializer import serializer
+from oldaplib.src.objectfactory import ResourceInstanceFactory
 from oldaplib.src.project import Project
 from oldaplib.src.propertyclass import PropClassAttrContainer, PropertyClass
 from oldaplib.src.enums.owlpropertytype import OwlPropertyType
@@ -118,21 +119,19 @@ class TestResourceClass(unittest.TestCase):
         cls._context['test'] = NamespaceIRI("http://oldap.org/test#")
         cls._context.use('test', 'dcterms')
 
-        cls._connection = Connection(server='http://localhost:7200',
-                                     userId="rosenth",
+        cls._connection = Connection(userId="rosenth",
                                      credentials="RioGrande",
-                                     repo="oldap",
                                      context_name="DEFAULT")
 
-        cls._unpriv = Connection(server='http://localhost:7200',
-                                 repo="oldap",
-                                 userId="fornaro",
+        cls._unpriv = Connection(userId="fornaro",
                                  credentials="RioGrande",
                                  context_name="DEFAULT")
 
 
         cls._connection.clear_graph(Xsd_QName('test:shacl'))
         cls._connection.clear_graph(Xsd_QName('test:onto'))
+        cls._connection.clear_graph(Xsd_QName('test:data'))
+
         cls._connection.clear_graph(Xsd_QName('dcterms:shacl'))
         cls._connection.clear_graph(Xsd_QName('dcterms:onto'))
 
@@ -792,6 +791,110 @@ class TestResourceClass(unittest.TestCase):
         s = set(r2.superclass.keys())
         self.assertEqual({"oldap:Thing", "test:testMyResMinimal", 'http://www.cidoc-crm.org/cidoc-crm/E22_Man-Made_Object'}, s)
 
+    def test_updating_superclass_add(self):
+        r1 = ResourceClass(con=self._connection,
+                           project=self._project,
+                           owlclass_iri=Iri("test:Superclass1"),
+                           label=LangString(["Superclass1@en", "Superclass1@fr"]),
+                           closed=Xsd_boolean(True))
+        r1.create()
+
+        r2 = ResourceClass(con=self._connection,
+                           project=self._project,
+                           owlclass_iri=Iri("test:Superclass2"),
+                           label=LangString(["Superclass2@en", "Superclass2@fr"]),
+                           closed=Xsd_boolean(True))
+        r2.create()
+
+        r3 = ResourceClass(con=self._connection,
+                           project=self._project,
+                           owlclass_iri=Iri("test:Superclass3"),
+                           label=LangString(["Superclass2@en", "Superclass2@fr"]),
+                           closed=Xsd_boolean(True))
+        r3.create()
+
+        p1 = PropertyClass(con=self._connection,
+                           project=self._project,
+                           property_class_iri=Iri('test:thingtest_prop1'),
+                           datatype=XsdDatatypes.string,
+                           name=LangString(["ThingTestProp1"]))
+        hasproperties: list[HasProperty] = [
+            HasProperty(con=self._connection, project=self._project, prop=p1, order=1)
+        ]
+        r = ResourceClass(con=self._connection,
+                           project=self._project,
+                           owlclass_iri=Iri("test:ResWithSuperThingAdd"),
+                           superclass={'http://www.cidoc-crm.org/cidoc-crm/E22_Man-Made_Object', "test:Superclass1"},
+                           label=LangString(["CreateResTest@en", "CréationResTeste@fr"]),
+                           comment=LangString("For testing purposes@en"),
+                           closed=Xsd_boolean(True),
+                           hasproperties=hasproperties)
+        r.create()
+        r = ResourceClass.read(con=self._connection,
+                                project=self._project,
+                                owl_class_iri=Iri('test:ResWithSuperThingAdd'))
+        r.add_superclasses({'test:Superclass2', 'test:Superclass3'})
+        r.update()
+        r = ResourceClass.read(con=self._connection,
+                                project=self._project,
+                                owl_class_iri=Iri('test:ResWithSuperThingAdd'))
+        s = set(r.superclass.keys())
+        self.assertEqual(s, {"oldap:Thing", 'http://www.cidoc-crm.org/cidoc-crm/E22_Man-Made_Object', "test:Superclass1", "test:Superclass2", "test:Superclass3"})
+
+    def test_updating_superclass_del(self):
+        r1 = ResourceClass(con=self._connection,
+                           project=self._project,
+                           owlclass_iri=Iri("test:SuperclassDel1"),
+                           label=LangString(["SuperclassDel1@en", "SuperclassDel1@fr"]),
+                           closed=Xsd_boolean(True))
+        r1.create()
+
+        r2 = ResourceClass(con=self._connection,
+                           project=self._project,
+                           owlclass_iri=Iri("test:SuperclassDel2"),
+                           label=LangString(["SuperclassDel2@en", "SuperclassDel2@fr"]),
+                           closed=Xsd_boolean(True))
+        r2.create()
+
+        r3 = ResourceClass(con=self._connection,
+                           project=self._project,
+                           owlclass_iri=Iri("test:SuperclassDel3"),
+                           label=LangString(["SuperclassDel3@en", "SuperclassDel3@fr"]),
+                           closed=Xsd_boolean(True))
+        r3.create()
+
+        p1 = PropertyClass(con=self._connection,
+                           project=self._project,
+                           property_class_iri=Iri('test:deltest_prop1'),
+                           datatype=XsdDatatypes.string,
+                           name=LangString(["DelTestProp1"]))
+        hasproperties: list[HasProperty] = [
+            HasProperty(con=self._connection, project=self._project, prop=p1, order=1)
+        ]
+        r = ResourceClass(con=self._connection,
+                           project=self._project,
+                           owlclass_iri=Iri("test:ResWithManySuper2"),
+                           superclass={
+                               'http://www.cidoc-crm.org/cidoc-crm/E22_Man-Made_Object',
+                               "test:SuperclassDel1",
+                               "test:SuperclassDel2",
+                               "test:SuperclassDel3"},
+                           label=LangString(["CreateResTestDelSup@en", "CréationResTesteDelSup@fr"]),
+                           comment=LangString("For testing purposes@en"),
+                           closed=Xsd_boolean(True),
+                           hasproperties=hasproperties)
+        r.create()
+        r = ResourceClass.read(con=self._connection,
+                                project=self._project,
+                                owl_class_iri=Iri("test:ResWithManySuper2"))
+        r.del_superclasses({'test:SuperclassDel1', 'test:SuperclassDel3'})
+        r.update()
+        r = ResourceClass.read(con=self._connection,
+                                project=self._project,
+                                owl_class_iri=Iri("test:ResWithManySuper2"))
+        s = set(r.superclass.keys())
+        self.assertEqual(s, {"oldap:Thing", 'http://www.cidoc-crm.org/cidoc-crm/E22_Man-Made_Object', "test:SuperclassDel2"})
+
     def test_creating_with_thing_sc(self):
         p1 = PropertyClass(con=self._connection,
                            project=self._project,
@@ -1142,7 +1245,7 @@ class TestResourceClass(unittest.TestCase):
                                 owl_class_iri=Iri('test:CrazyB'))
         self.assertEqual({Iri("oldap:Thing"), Iri('http://www.cidoc-crm.org/cidoc-crm/E22_Man-Made_Object')}, set(r1.superclass))
 
-    def test_updateing_sc_C(self):
+    def test_updating_sc_C(self):
         p1 = PropertyClass(con=self._connection,
                            project=self._project,
                            property_class_iri=Iri('test:p3'),
@@ -1175,7 +1278,7 @@ class TestResourceClass(unittest.TestCase):
                           'http://www.cidoc-crm.org/cidoc-crm/E22_Man-Made_Object',
                           'http://gugus.com/gaga/wird/nicht/gehen'}, set(r1.superclass))
 
-    def test_updateing_sc_D(self):
+    def test_updating_sc_D(self):
         p1 = PropertyClass(con=self._connection,
                            project=self._project,
                            property_class_iri=Iri('test:p4'),
@@ -1201,7 +1304,7 @@ class TestResourceClass(unittest.TestCase):
                                 owl_class_iri=Iri('test:CrazyD'))
         self.assertIsNone(r1[ResClassAttribute.LABEL])
 
-    def test_updateing_sc_E(self):
+    def test_updating_sc_E(self):
         p1 = PropertyClass(con=self._connection,
                            project=self._project,
                            property_class_iri=Iri('test:p5'),
@@ -1227,6 +1330,38 @@ class TestResourceClass(unittest.TestCase):
                                 owl_class_iri=Iri('test:CrazyE'))
         self.assertIsNone(r1[ResClassAttribute.LABEL])
 
+    def test_updating_sc_F(self):
+        r1 = ResourceClass(con=self._connection,
+                           project=self._project,
+                           owlclass_iri=Iri("test:CrazyF"),
+                           label=LangString(["LabelF english@en", "Label F french@fr"]),
+                           comment=LangString("commentF english@en"))
+        r1.create()
+        r1 = ResourceClass.read(con=self._connection,
+                                project=self._project,
+                                owl_class_iri=Iri('test:CrazyF'))
+        del r1.label[Language.EN]
+        r1.label.add("Label F italian@it")
+        r1.update()
+        r1 = ResourceClass.read(con=self._connection,
+                                project=self._project,
+                                owl_class_iri=Iri('test:CrazyF'))
+        self.assertEqual(r1.label, LangString(["Label F french@fr", "Label F italian@it"]))
+        #
+        # Testing if also OWL ontology has been updated
+        #
+        sparql = self._context.sparql_context
+        sparql += """
+        SELECT ?label
+        FROM test:onto
+        WHERE {{
+            test:CrazyF rdfs:label ?label .
+        }}
+        """
+        jsonres = self._connection.query(sparql)
+        res = QueryProcessor(self._context, jsonres)
+        for r in res:
+            self.assertIn(r['label'], [Xsd_string("Label F french@fr"), Xsd_string("Label F italian@it")])
 
     # @unittest.skip('Work in progress')
     def test_delete_props(self):
@@ -1347,6 +1482,63 @@ class TestResourceClass(unittest.TestCase):
                                         owl_class_iri=Iri("test:testMyResMinimal"))
         self.assertEqual(Iri("test:testMyResMinimal"), superclass.owl_class_iri)
 
+    def test_in_use_case(self):
+        p1 = PropertyClass(con=self._connection,
+                           project=self._project,
+                           property_class_iri=Iri('test:prop_A'),
+                           datatype=XsdDatatypes.string,
+                           name=LangString(["Test property A@en", "Testprädikat A@de"]),
+                           description=LangString("A property for testing...@en"))
+
+        hasproperties: list[HasProperty] = [
+            HasProperty(con=self._connection, project=self._project, prop=p1),
+        ]
+        r1 = ResourceClass(con=self._connection,
+                           project=self._project,
+                           owlclass_iri=Iri("test:TestResourceInUse"),
+                           superclass="test:testMyResMinimal",
+                           label=LangString(["CreateResTest@en", "CréationResTeste@fr"]),
+                           comment=LangString("For testing purposes@en"),
+                           closed=Xsd_boolean(True),
+                           hasproperties=hasproperties)
+        r1.create()
+
+        factory = ResourceInstanceFactory(con=self._connection, project=self._project)
+        TestResourceInUse = factory.createObjectInstance('TestResourceInUse')
+        data = TestResourceInUse(
+            label=['lanel_aaa@en', 'label_bbb@de'],
+            comment=['comment_aaa@en', 'comment_bbb@de'],
+            prop_A="WAELIWAS SOLL DENN DAS?",
+            grantsPermission=Iri('oldap:GenericView')
+        )
+        data.create()
+        data2 = TestResourceInUse.read(con=self._connection,
+                                       project='test',
+                                       iri=data.iri)
+
+        r1 = ResourceClass.read(con=self._connection,
+                                project=self._project,
+                                owl_class_iri=Iri("test:TestResourceInUse"), ignore_cache=True)
+        with self.assertRaises(OldapErrorInUse):
+            r1.delete()
+
+        r1.del_superclasses("test:testMyResMinimal")
+        with self.assertRaises(OldapErrorInUse):
+            r1.update()
+
+        r1 = ResourceClass.read(con=self._connection,
+                                project=self._project,
+                                owl_class_iri=Iri("test:TestResourceInUse"), ignore_cache=True)
+        r1.add_superclasses("dcterms:waseliwas")
+        with self.assertRaises(OldapErrorInUse):
+            r1.update()
+
+
+
+
+
+
+
     # @unittest.skip('Work in progress')
     def test_write_trig(self):
         project_id = PropertyClass(con=self._connection,
@@ -1382,6 +1574,12 @@ class TestResourceClass(unittest.TestCase):
                            comment=LangString(["Definiton of a project@en", "Definition eines Projektes@de"]),
                            closed=Xsd_boolean(True),
                            hasproperties=hasproperties)
+        r1.create()
+        r1.label.add("projet@fr")
+        r1.update()
+        r1.read(con=self._connection,
+                project=self._project,
+                owl_class_iri=Iri("test:Project"), ignore_cache=True)
         r1.write_as_trig("gaga.trig")
 
 
